using Jose;

namespace FitConnect.Encryption;

public class JoseEncryptor : IEncryptor {
    private const JweEncryption Encryption = JweEncryption.A256GCM;
    private const JweCompression Compression = JweCompression.DEF;
    private const JweAlgorithm Algorithm = JweAlgorithm.RSA_OAEP;

    private const string ErrorMessage =
        "On macOS add DYLD_LIBRARY_PATH to your environment variables. Look at the README for more information.";

    private (string plainText, byte[] plainBytes, byte[] tag) Decrypt(Jwk key, string payload) {
        var result = JWE.Decrypt(payload, key);

        return (result.Plaintext, result.PlaintextBytes, result.AuthTag);
    }


    public (string plainText, byte[] plainBytes, byte[] tag) Decrypt(string key, string cipher) {
        try {
            var jwk = Jwk.FromJson(key, new Jose.JsonMapper());
            return Decrypt(jwk, cipher);
        }
        catch (PlatformNotSupportedException e) {
            throw new PlatformNotSupportedException(ErrorMessage, e);
        }
    }

    public string Encrypt(string key, string plain) {
        try {
            var jwk = Jwk.FromJson(key, new Jose.JsonMapper());
            return JWE.Encrypt(plain,
                new JweRecipient[] { new JweRecipient(Algorithm, jwk) },
                Encryption, compression: Compression);
        }
        catch (PlatformNotSupportedException e) {
            throw new PlatformNotSupportedException(ErrorMessage, e);
        }
    }

    public string Encrypt(string key, byte[] plain) {
        try {
            var jwk = Jwk.FromJson(key, new Jose.JsonMapper());
            return JWE.EncryptBytes(plain,
                new JweRecipient[] { new JweRecipient(Algorithm, jwk) },
                Encryption, compression: Compression);
        }
        catch (PlatformNotSupportedException e) {
            throw new PlatformNotSupportedException(ErrorMessage, e);
        }
    }
}
