using System.Text.Json;
using FitConnect.RestService;
using FitConnect.Services.Models;
using FitConnect.Services.Models.v1;

namespace FitConnect.Services;

public class SubmissionService : RestCallService, ISubmissionService {
    private const string ApiVersion = "v1";

    public SubmissionService(string baseUrl) : base($"{baseUrl}/{ApiVersion}") {
    }

    /// <summary>
    ///     <para>@PostMapping("/submissions")</para>
    /// </summary>
    /// <param name="submissionDto">RequestBody</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public SubmissionCreatedDto CreateSubmission(CreateSubmissionDto submissionDto) {
        var result = RestCall<SubmissionCreatedDto>(
            "/submissions",
            HttpMethod.Post,
            JsonSerializer.Serialize(submissionDto)).Result;

        if (result == null)
            throw new ArgumentException("Submission creation failed");

        return result;
    }


    /// <summary>
    ///     <para>
    ///         @PutMapping(value = "/submissions/{submissionId}/attachments/{attachmentId}", consumes =
    ///         "application/jose")
    ///     </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="attachmentId">PathVariable</param>
    /// <param name="encryptedAttachmentContent">RequestBody</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public bool AddSubmissionAttachment(string submissionId, string attachmentId,
        string encryptedAttachmentContent) {
        var result = RestCallForString(
            $"/submissions/{submissionId}/attachments/{attachmentId}",
            HttpMethod.Put,
            encryptedAttachmentContent,
            "application/jose").Result;
        return true;
    }

    /// <summary>
    ///     <para>@PutMapping(value = "/submissions/{submissionId}", consumes = "application/json") </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="submitSubmission">RequestBody</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public SubmissionReducedDto SubmitSubmission(string submissionId,
        SubmitSubmissionDto submitSubmission) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     <para>@GetMapping("/submissions")</para>
    /// </summary>
    /// <param name="destinationId">RequestParam</param>
    /// <param name="offset">RequestParam</param>
    /// <param name="limit">RequestParam</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public async Task<SubmissionsForPickupDto> ListSubmissions(string destinationId, int offset,
        int limit) {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     <para>@GetMapping("/submissions/{submissionId}")</para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public SubmissionDto GetSubmission(string submissionId) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     <para>
    ///         @GetMapping(value = "/submissions/{submissionId}/attachments/{attachmentId}", produces =
    ///         "application/jose")
    ///     </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="attachmentId">PathVariable</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public string GetAttachment(string submissionId, string attachmentId) {
        throw new NotImplementedException();
    }

    public string GetKey(string keyId) {
        throw new NotImplementedException();
    }
}
